%% script2


%% load data and calc extrinsics

clear
close all
load cpts
load mpts
load CalibResults

worldPoints = [ 0 0; 21 0; 21 21; 0 21];

[R1, T1] = extrinsics(cpts1(1:4,:),worldPoints,cameraParams)

[R2, T2] = extrinsics(cpts2(1:4,:),worldPoints,cameraParams)

save extrinsics R1 T1 R2 T2

%% camera origins

A1 = [R1 [0 0 0]'; T1 1];
A2 = [R2 [0 0 0]'; T2 1];

C1 = -T1*R1'

C2 = -T2*R2'

%% intrinsic matrix

fc = cameraParams.FocalLength;
cc = cameraParams.PrincipalPoint;
K = [fc(1) 0 cc(1); 0 fc(2) cc(2); 0 0 1]';
Kinv = inv(K);

%% check world origin -> camera 

cam = T1/T1(3)

format shortg
img = cam*K

measured = cpts1(1,:)
format

%% find direction to project

for n=1:4
    k1c = [cpts1(n,:) 1]*Kinv;
    k2c = [cpts2(n,:) 1]*Kinv;
    
    k1 = k1c*R1';
    k2 = k2c*R2';
    
    % find projection distances
    
    b = [ dot(C2-C1,k1); -dot(C2-C1,k2)];
    A = [dot(k1,k1) -dot(k1,k2); -dot(k1,k2) dot(k2,k2)];
    q = A\b;
    
    pA = C1+k1*q(1);
    pB = C2+k2*q(2);
    % fprintf('norm(pA-pB) = %g\n',norm(pA-pB)); % should be the same
    
    fprintf('%11g \t %11g \t %11g  \t world %g %g 0\n',pA,worldPoints(n,:));
end




%% show results

wpts = [ 0 0 0; 21 0 0; 21 21 0; 0 21 0];

n = 1;
ndx = [1:4 1];
plot3(wpts(ndx,1),-wpts(ndx,2),-wpts(ndx,3),'k','LineWidth',2);
hold on
plot3(wpts(n,1),-wpts(n,2),-wpts(1,3),'ko','MarkerSize',9);
plot3(C1(1),-C1(2),-C1(3),'ro','MarkerSize',9);
plot3(C2(1),-C2(2),-C2(3),'ro','MarkerSize',9);
k1c = 5*[cpts1(n,:) 1 ]*Kinv;
k1c = [k1c 1]*inv(A1);
plot3(k1c(1),-k1c(2),-k1c(3),'rx','MarkerSize',7);
k2c = 5*[cpts2(n,:) 1]*Kinv;
k2c = [k2c 1]*inv(A2);
plot3(k2c(1),-k2c(2),-k2c(3),'rx','MarkerSize',7);


clear dst

k1c = [cpts1(n,:) 1 ]*Kinv;
k2c = [cpts2(n,:) 1]*Kinv;

k1 = k1c*R1';
k2 = k2c*R2';

% find projection distances

b = [ dot(C2-C1,k1); -dot(C2-C1,k2)];
A = [dot(k1,k1) -dot(k1,k2); -dot(k1,k2) dot(k2,k2)];
q = A\b;

dst(1,:) = C1;
dst(2,:) = C1 + q(1)*k1;
plot3(dst(:,1),-dst(:,2),-dst(:,3),'k--','LineWidth',2);
dst(1,:) = C2;
dst(2,:) = C2 + q(2)*k2;
plot3(dst(:,1),-dst(:,2),-dst(:,3),'k--','LineWidth',2);

hold off
xlabel('x');
ylabel('y');
grid
axis equal





