%% script3


%% load data and calc extrinsics

clear
close all
load cpts
load mpts
load CalibResults

worldPoints = [ 0 0; 21 0; 21 21; 0 21];

[R1, T1] = extrinsics(cpts1(1:4,:),worldPoints,cameraParams)

[R2, T2] = extrinsics(cpts2(1:4,:),worldPoints,cameraParams)

%% camera origins

C1 = -T1*R1'

C2 = -T2*R2'

%% intrinsic matrix

fc = cameraParams.FocalLength;
cc = cameraParams.PrincipalPoint;
K = [fc(1) 0 cc(1); 0 fc(2) cc(2); 0 0 1]'
Kinv = inv(K);

%% find projections

fprintf('\n');
npts = size(mpts1,1);
for n=1:24
    k1c = [mpts1(n,:) 1]*Kinv;
    k2c = [mpts2(n,:) 1]*Kinv;
    
    k1 = k1c*R1';
    k2 = k2c*R2';
    
    % find projection distance
    
    b = [ dot(C2-C1,k1); -dot(C2-C1,k2)];
    A = [dot(k1,k1) -dot(k1,k2); -dot(k1,k2) dot(k2,k2)];
    q = A\b;
    
    pA = C1+k1*q(1);
    pB = C2+k2*q(2);
    fprintf('%d \t %8.4f \t %8.4f \t %8.4f \t   %10.6f\n',n,pA,norm(pA-pB)); 
end

%%

figure(1)
rgb = imread(files{1});
show_points(rgb,mpts1);

%%

figure(2)
rgb = imread(files{2});
show_points(rgb,mpts2);

